AND = "AND"
COMMA = ","
DESCENDING = "DESC"
EQUALS_SIGN = "="
SOURCE_PREFIX = "S."
TARGET_PREFIX = "T."
WHITE_SPACE = " "
SQL_QUOTATION = "`"
QUESTION_MARK = "?"
OPEN_PARENTHESIS = "("
CLOSE_PARENTHESIS = ")"
DOUBLE_QUOTE = '"'
DOT = "."
