ASYNC_PATH = "async"
SYNC_PATH = "hub"
DATA_PATH = "data"
ASYNC_JOB_RESULT_PATH = "/{request_id}/results"
ASYNC_JOB_STATUS_PATH = "/{request_id}/status"
AUTH_URL = "https://{subdomain}.auth.marketingcloudapis.com/"
DATA_EXTENSION_KEY_PATH = "/dataextensions/key:{key}/rows"
DATA_EVENT_KEY_PATH = "/dataevents/key:{key}/rowset"
CUSTOM_OBJECT_DATA_PATH = "/customobjectdata/key/{key}/rowset"
TOKEN_PATH = "token"
GRANT_TYPE = "client_credentials"
FILTER_QUERY = "$filter={filter}"
V1 = "v1"
V2 = "v2"
FORWARD_SLASH = "/"
